/****************************************************************************
 *   $Id:: wdt.h 5803 2010-12-03 00:30:03Z usb00423                         $
 *   Project: NXP LPC17xx WDT example
 *
 *   Description:
 *     This file contains WDT code header definition.
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
****************************************************************************/
#ifndef __WDT_H 
#define __WDT_H

#define WDEN		0//(0x1<<0)
#define WDRESET		1//(0x1<<1)
#define WDTOF		2//(0x1<<2)
#define WDINT		3//(0x1<<3)

//#define WDT_FEED_VALUE	0x003FFFFF
#define WDT_FEED_VALUE	0xFF



/*****************************************************************************
**                            End Of File
******************************************************************************/
//#include "LPC17xx.h"
//#include "type.h"
//#include "timer.h"
//#include "wdt.h"

volatile uint32_t wdt_counter;

/*****************************************************************************
** Function name:		WDT_IRQHandler
**
** Descriptions:		Watchdog timer interrupt handler
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void WDT_IRQHandler(void)
{
  cbi(WDMOD,WDTOF);//WDTOF		/* clear the time-out terrupt flag */
  wdt_counter++;
  return;
}

/*****************************************************************************
** Function name:		WDTInit
**
** Descriptions:		Initialize watchdog timer, install the
**				watchdog timer interrupt handler
**
** parameters:			None
** Returned value:		true or false, return false if the VIC table
**				is full and WDT interrupt handler can be
**				installed.
**
*****************************************************************************/
void WDTInit( void )
{
  wdt_counter = 0;

  NVIC_EnableIRQ(WDT_IRQn);
  cbi(WDCLKSEL,0);
  cbi(WDCLKSEL,1);
  WDTC = WDT_FEED_VALUE;	/* once WDEN is set, the WDT will start after feeding */
  sbi(WDMOD,WDEN);
  sbi(WDMOD,WDRESET);

  WDFEED = 0xAA;		/* Feeding sequence */
  WDFEED = 0x55;
 return;
}

/*****************************************************************************
** Function name:		WDTFeed
**
** Descriptions:		Feed watchdog timer to prevent it from timeout
**
** parameters:			None
** Returned value:		None
**
*****************************************************************************/
void WDTFeed( void )
{
  WDFEED = 0xAA;		/* Feeding sequence */
  WDFEED = 0x55;
  return;
}

/******************************************************************************
**                            End Of File
******************************************************************************/
#endif /* end __WDT_H */
