/****************************************************************************
 *   $Id:: i2c.h 5865 2010-12-08 21:42:21Z usb00423                         $
 *   Project: NXP LPC17xx I2C example
 *
 *   Description:
 *     This file contains I2C code header definition.
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
****************************************************************************/
#ifndef __I2C_H 
#define __I2C_H

uint8_t   	SLAVE_ADR   =      	0xd0;
uint8_t 	PORT_USED	=		1;

/* If I2C SEEPROM is tested, make sure FAST_MODE_PLUS is 0.
For board to board test, this flag can be turned on. */
#define FAST_MODE_PLUS      0

#define I2C_PORT_NUM        3//drei ports, i2c0,i2c1 u.i2c2
#define BUFSIZE             64
#define MAX_TIMEOUT         0x00FFFFFF

static LPC_I2C_TypeDef (* const LPC_I2C[I2C_PORT_NUM]) = { LPC_I2C0, LPC_I2C1, LPC_I2C2 };

#define I2CMASTER           0x01
#define I2CSLAVE            0x02

#define READ_WRITE          0x01

#define RD_BIT              0x01

#define I2C_IDLE              0
#define I2C_STARTED           1
#define I2C_RESTARTED         2
#define I2C_REPEATED_START    3
#define DATA_ACK              4
#define DATA_NACK             5
#define I2C_BUSY              6
#define I2C_NO_DATA           7
#define I2C_NACK_ON_ADDRESS   8
#define I2C_NACK_ON_DATA      9
#define I2C_ARBITRATION_LOST  10
#define I2C_TIME_OUT          11
#define I2C_OK                12

#define I2CONSET_I2EN       (0x1<<6)  /* I2C Control Set Register */
#define I2CONSET_AA         (0x1<<2)
#define I2CONSET_SI         (0x1<<3)
#define I2CONSET_STO        (0x1<<4)
#define I2CONSET_STA        (0x1<<5)

#define I2CONCLR_AAC        (0x1<<2)  /* I2C Control clear Register */
#define I2CONCLR_SIC        (0x1<<3)
#define I2CONCLR_STAC       (0x1<<5)
#define I2CONCLR_I2ENC      (0x1<<6)

#define I2DAT_I2C			0x00000000  /* I2C Data Reg */
#define I2ADR_I2C			0x00000000  /* I2C Slave Address Reg */
//#define I2SCLH_SCLH			0x00000080  /* I2C SCL Duty Cycle High Reg */
//#define I2SCLL_SCLL			0x00000080  /* I2C SCL Duty Cycle Low Reg */
#define I2SCLH_SCLH			125  /* I2C SCL Duty Cycle High Reg */
#define I2SCLL_SCLL			125  /* I2C SCL Duty Cycle Low Reg */
#define I2SCLH_HS_SCLH		0x00000008  /* Fast Plus I2C SCL Duty Cycle High Reg */
#define I2SCLL_HS_SCLL		0x00000008  /* Fast Plus I2C SCL Duty Cycle Low Reg */

extern void I2C0_IRQHandler( void );
extern void I2C1_IRQHandler( void );
extern void I2C2_IRQHandler( void );
extern void I2C0Init( void );
extern void I2C1Init( void );
extern void I2C2Init( void );
extern uint32_t I2CStart( uint32_t portNum );
extern uint32_t I2CStop( uint32_t portNum );
extern uint32_t I2CEngine( uint32_t portNum );



/****************************************************************************
 *   $Id:: i2c.c 5865 2010-12-08 21:42:21Z usb00423                         $
 *   Project: NXP LPC17xx I2C example
 *
 *   Description:
 *     This file contains I2C code example which include I2C initialization, 
 *     I2C interrupt handler, and APIs for I2C access.
 *
 ****************************************************************************
 * Software that is described herein is for illustrative purposes only
 * which provides customers with programming information regarding the
 * products. This software is supplied "AS IS" without any warranties.
 * NXP Semiconductors assumes no responsibility or liability for the
 * use of the software, conveys no license or title under any patent,
 * copyright, or mask work right to the product. NXP Semiconductors
 * reserves the right to make changes in the software without
 * notification. NXP Semiconductors also make no representation or
 * warranty that such application will be suitable for the specified
 * use without further testing or modification.
****************************************************************************/
#include "lpc17xx.h"
#include "type.h"
//#include "i2c.h"



volatile uint32_t I2CMasterState[I2C_PORT_NUM] = {I2C_IDLE,I2C_IDLE,I2C_IDLE};
volatile uint32_t timeout[I2C_PORT_NUM] = {0, 0, 0};

volatile uint8_t I2CMasterBuffer[I2C_PORT_NUM][BUFSIZE];//je 64 bytes pro port
volatile uint8_t I2CSlaveBuffer[I2C_PORT_NUM][BUFSIZE];
volatile uint32_t I2CCount[I2C_PORT_NUM] = {0, 0, 0};
volatile uint32_t I2CReadLength[I2C_PORT_NUM];
volatile uint32_t I2CWriteLength[I2C_PORT_NUM];

volatile uint32_t RdIndex0 = 0, RdIndex1 = 0, RdIndex2 = 0;
volatile uint32_t WrIndex0 = 0, WrIndex1 = 0, WrIndex2 = 0;

/* 
From device to device, the I2C communication protocol may vary, 
in the example below, the protocol uses repeated start to read data from or 
write to the device:
For master read: the sequence is: STA,Addr(W),offset,RE-STA,Addr(r),data...STO 
for master write: the sequence is: STA,Addr(W),offset,RE-STA,Addr(w),data...STO
Thus, in state 8, the address is always WRITE. in state 10, the address could 
be READ or WRITE depending on the I2C command.
*/   

/*****************************************************************************
** Function name:		I2C_IRQHandler
**
** Descriptions:		I2C interrupt handler, deal with master mode only.
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void I2C0_IRQHandler(void) 
{
  uint8_t StatValue;

  timeout[0] = 0;
  /* this handler deals with master read and master write only */
  StatValue = LPC_I2C0->I2STAT;
  switch ( StatValue )
  {
	case 0x08:			/* A Start condition is issued. */
	WrIndex0 = 0;
	LPC_I2C0->I2DAT = I2CMasterBuffer[0][WrIndex0++];
	LPC_I2C0->I2CONCLR  = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x10:			/* A repeated started is issued */
	RdIndex0 = 0;
	/* Send SLA with R bit set, */
	LPC_I2C0->I2DAT = I2CMasterBuffer[0][WrIndex0++];
	LPC_I2C0->I2CONCLR = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x18:			/* Regardless, it's a ACK */
	if ( I2CWriteLength[0] == 1 )
	{
	  LPC_I2C0->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
	  I2CMasterState[0] = I2C_NO_DATA;
	}
	else
	{
	  LPC_I2C0->I2DAT = I2CMasterBuffer[0][WrIndex0++];
	}
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x28:	/* Data byte has been transmitted, regardless ACK or NACK */
	if ( WrIndex0 < I2CWriteLength[0] )
	{   
	  LPC_I2C0->I2DAT = I2CMasterBuffer[0][WrIndex0++]; /* this should be the last one */
	}
	else
	{
	  if ( I2CReadLength[0] != 0 )
	  {
		LPC_I2C0->I2CONSET = I2CONSET_STA;	/* Set Repeated-start flag */
	  }
	  else
	  {
		LPC_I2C0->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
		I2CMasterState[0] = I2C_OK;
	  }
	}
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x30:
	LPC_I2C0->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
	I2CMasterState[0] = I2C_NACK_ON_DATA;
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x40:	/* Master Receive, SLA_R has been sent */
	if ( (RdIndex0 + 1) < I2CReadLength[0] )
	{
	  /* Will go to State 0x50 */
	  LPC_I2C0->I2CONSET = I2CONSET_AA;	/* assert ACK after data is received */
	}
	else
	{
	  /* Will go to State 0x58 */
	  LPC_I2C0->I2CONCLR = I2CONCLR_AAC;	/* assert NACK after data is received */
	}
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x50:	/* Data byte has been received, regardless following ACK or NACK */
	I2CSlaveBuffer[0][RdIndex0++] = LPC_I2C0->I2DAT;
	if ( (RdIndex0 + 1) < I2CReadLength[0] )
	{   
	  LPC_I2C0->I2CONSET = I2CONSET_AA;	/* assert ACK after data is received */
	}
	else
	{
	  LPC_I2C0->I2CONCLR = I2CONCLR_AAC;	/* assert NACK on last byte */
	}
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x58:
	I2CSlaveBuffer[0][RdIndex0++] = LPC_I2C0->I2DAT;
	I2CMasterState[0] = I2C_OK;
	LPC_I2C0->I2CONSET = I2CONSET_STO;	/* Set Stop flag */
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;	/* Clear SI flag */
	break;

	case 0x20:		/* regardless, it's a NACK */
	case 0x48:
	LPC_I2C0->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
	I2CMasterState[0] = I2C_NACK_ON_ADDRESS;
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x38:		/* Arbitration lost, in this example, we don't
					deal with multiple master situation */
	default:
	I2CMasterState[0] = I2C_ARBITRATION_LOST;
	LPC_I2C0->I2CONCLR = I2CONCLR_SIC;
	break;
  }
  return;
}

/*****************************************************************************
** Function name:		I2C_IRQHandler
**
** Descriptions:		I2C interrupt handler, deal with master mode only.
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
v u8 ic2_scan_found=2;
void I2C1_IRQHandler(void) 
{
  uint8_t StatValue;

  timeout[1] = 0;
  /* this handler deals with master read and master write only */
  StatValue = LPC_I2C1->I2STAT;
  switch ( StatValue )
  {
	case 0x08:			/* A Start condition is issued. */
	WrIndex1 = 0;
	LPC_I2C1->I2DAT = I2CMasterBuffer[1][WrIndex1++];
	LPC_I2C1->I2CONCLR = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x10:			/* A repeated started is issued */
	RdIndex1 = 0;
	/* Send SLA with R bit set, */
	LPC_I2C1->I2DAT = I2CMasterBuffer[1][WrIndex1++];
	LPC_I2C1->I2CONCLR = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x18:			/* Regardless, it's a ACK */
ic2_scan_found=0;
	if ( I2CWriteLength[1] == 1 )
	{
	  LPC_I2C1->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
	  I2CMasterState[1] = I2C_NO_DATA;
	}
	else
	{
	  LPC_I2C1->I2DAT = I2CMasterBuffer[1][WrIndex1++];
	}
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x28:	/* Data byte has been transmitted, regardless ACK or NACK */
	if ( WrIndex1 < I2CWriteLength[1] )
	{   
	  LPC_I2C1->I2DAT = I2CMasterBuffer[1][WrIndex1++]; /* this should be the last one */
	}
	else
	{
	  if ( I2CReadLength[1] != 0 )
	  {
		LPC_I2C1->I2CONSET = I2CONSET_STA;	/* Set Repeated-start flag */
	  }
	  else
	  {
		LPC_I2C1->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
		I2CMasterState[1] = I2C_OK;
	  }
	}
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x30:
	LPC_I2C1->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
	I2CMasterState[1] = I2C_NACK_ON_DATA;
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x40:	/* Master Receive, SLA_R has been sent */
	if ( (RdIndex1 + 1) < I2CReadLength[1] )
	{
	  /* Will go to State 0x50 */
	  LPC_I2C1->I2CONSET = I2CONSET_AA;	/* assert ACK after data is received */
	}
	else
	{
	  /* Will go to State 0x58 */
	  LPC_I2C1->I2CONCLR = I2CONCLR_AAC;	/* assert NACK after data is received */
	}
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x50:	/* Data byte has been received, regardless following ACK or NACK */
	I2CSlaveBuffer[1][RdIndex1++] = LPC_I2C1->I2DAT;
	if ( (RdIndex1 + 1) < I2CReadLength[1] )
	{   
	  LPC_I2C1->I2CONSET = I2CONSET_AA;	/* assert ACK after data is received */
	}
	else
	{
	  LPC_I2C1->I2CONCLR = I2CONCLR_AAC;	/* assert NACK on last byte */
	}
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x58:
	I2CSlaveBuffer[1][RdIndex1++] = LPC_I2C1->I2DAT;
	I2CMasterState[1] = I2C_OK;
	LPC_I2C1->I2CONSET = I2CONSET_STO;	/* Set Stop flag */
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;	/* Clear SI flag */
	break;

	case 0x20:		/* regardless, it's a NACK */
	case 0x48:
	LPC_I2C1->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
	I2CMasterState[1] = I2C_NACK_ON_ADDRESS;
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x38:		/* Arbitration lost, in this example, we don't
					deal with multiple master situation */
ic2_scan_found=1;

	default:
	I2CMasterState[1] = I2C_ARBITRATION_LOST;
	LPC_I2C1->I2CONCLR = I2CONCLR_SIC;
	break;
  }
  return;
}

/*****************************************************************************
** Function name:		I2C_IRQHandler
**
** Descriptions:		I2C interrupt handler, deal with master mode only.
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void I2C2_IRQHandler(void) 
{
  uint8_t StatValue;

  timeout[2] = 0;
  /* this handler deals with master read and master write only */
  StatValue = LPC_I2C2->I2STAT;
  switch ( StatValue )
  {
	case 0x08:			/* A Start condition is issued. */
	WrIndex2 = 0;
	LPC_I2C2->I2DAT = I2CMasterBuffer[2][WrIndex2++];
	LPC_I2C2->I2CONCLR = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x10:			/* A repeated started is issued */
	RdIndex2 = 0;
	/* Send SLA with R bit set, */
	LPC_I2C2->I2DAT = I2CMasterBuffer[2][WrIndex2++];
	LPC_I2C2->I2CONCLR = (I2CONCLR_SIC | I2CONCLR_STAC);
	break;
	
	case 0x18:			/* Regardless, it's a ACK */
	if ( I2CWriteLength[2] == 1 )
	{
	  LPC_I2C2->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
	  I2CMasterState[2] = I2C_NO_DATA;
	}
	else
	{
	  LPC_I2C2->I2DAT = I2CMasterBuffer[2][WrIndex2++];
	}
	LPC_I2C2->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x28:	/* Data byte has been transmitted, regardless ACK or NACK */
	if ( WrIndex2 < I2CWriteLength[2] )
	{   
	  LPC_I2C2->I2DAT = I2CMasterBuffer[2][WrIndex2++]; /* this should be the last one */
	}
	else
	{
	  if ( I2CReadLength[2] != 0 )
	  {
		LPC_I2C2->I2CONSET = I2CONSET_STA;	/* Set Repeated-start flag */
	  }
	  else
	  {
		LPC_I2C2->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
		I2CMasterState[2] = I2C_OK;
	  }
	}
	LPC_I2C2->I2CONCLR = I2CONCLR_SIC;
	break;

	case 0x30:
	LPC_I2C2->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
	I2CMasterState[2] = I2C_NACK_ON_DATA;
	LPC_I2C2->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x40:	/* Master Receive, SLA_R has been sent */
	if ( (RdIndex2 + 1) < I2CReadLength[2] )
	{
	  /* Will go to State 0x50 */
	  LPC_I2C2->I2CONSET = I2CONSET_AA;	/* assert ACK after data is received */
	}
	else
	{
	  /* Will go to State 0x58 */
	  LPC_I2C2->I2CONCLR = I2CONCLR_AAC;	/* assert NACK after data is received */
	}
	LPC_I2C2->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x50:	/* Data byte has been received, regardless following ACK or NACK */
	I2CSlaveBuffer[2][RdIndex2++] = LPC_I2C2->I2DAT;
	if ( (RdIndex2 + 1) < I2CReadLength[2] )
	{   
	  LPC_I2C2->I2CONSET = I2CONSET_AA;	/* assert ACK after data is received */
	}
	else
	{
	  LPC_I2C2->I2CONCLR = I2CONCLR_AAC;	/* assert NACK on last byte */
	}
	LPC_I2C2->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x58:
	I2CSlaveBuffer[2][RdIndex2++] = LPC_I2C2->I2DAT;
	I2CMasterState[2] = I2C_OK;
	LPC_I2C2->I2CONSET = I2CONSET_STO;	/* Set Stop flag */
	LPC_I2C2->I2CONCLR = I2CONCLR_SIC;	/* Clear SI flag */
	break;

	case 0x20:		/* regardless, it's a NACK */
	case 0x48:
	LPC_I2C2->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
	I2CMasterState[2] = I2C_NACK_ON_ADDRESS;
	LPC_I2C2->I2CONCLR = I2CONCLR_SIC;
	break;
	
	case 0x38:		/* Arbitration lost, in this example, we don't
					deal with multiple master situation */
	default:
	I2CMasterState[2] = I2C_ARBITRATION_LOST;
	LPC_I2C2->I2CONCLR = I2CONCLR_SIC;
	break;
  }
  return;
}

/*****************************************************************************
** Function name:		I2CStart
**
** Descriptions:		Create I2C start condition, a timeout
**				value is set if the I2C never gets started,
**				and timed out. It's a fatal error. 
**
** parameters:			None
** Returned value:		true or false, return false if timed out
** 
*****************************************************************************/
uint32_t I2CStart( uint32_t portNum )
{
  uint32_t retVal = FALSE;
  timeout[portNum] = 0;
  /*--- Issue a start condition ---*/
  LPC_I2C[portNum]->I2CONSET = I2CONSET_STA;	/* Set Start flag */
  /*--- Wait until START transmitted ---*/
  while( 1 )
  {
	if ( I2CMasterState[portNum] == I2C_STARTED )
	{
	  retVal = TRUE;
	  break;	
	}
	if ( timeout[portNum] >= MAX_TIMEOUT )
	{
	  retVal = FALSE;
	  break;
	}
	timeout[portNum]++;
  }
  return( retVal );
}

/*****************************************************************************
** Function name:		I2CStop
**
** Descriptions:		Set the I2C stop condition, if the routine
**				never exit, it's a fatal bus error.
**
** parameters:			None
** Returned value:		true or never return
** 
*****************************************************************************/
uint32_t I2CStop( uint32_t portNum )
{
  LPC_I2C[portNum]->I2CONSET = I2CONSET_STO;      /* Set Stop flag */
  LPC_I2C[portNum]->I2CONCLR = I2CONCLR_SIC;  /* Clear SI flag */
            
  /*--- Wait for STOP detected ---*/
  while( LPC_I2C[portNum]->I2CONSET & I2CONSET_STO );
  return TRUE;
}

/*****************************************************************************
** Function name:		I2CInit
**
** Descriptions:		Initialize I2C controller as a master
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void I2C0Init( void ) 
{
 PCONP |= (1 << 7);

  /* set PIO0.27 and PIO0.28 to I2C0 SDA and SCL */
  /* function to 01 on both SDA and SCL. */
  PINSEL1 &= ~((0x03<<22)|(0x03<<24));
  PINSEL1 |= ((0x01<<22)|(0x01<<24));
 
  /*--- Clear flags ---*/
  LPC_I2C0->I2CONCLR = I2CONCLR_AAC | I2CONCLR_SIC | I2CONCLR_STAC | I2CONCLR_I2ENC;

  /*--- Reset registers ---*/
#if FAST_MODE_PLUS
  I2CPADCFG |= ((0x1<<0)|(0x1<<2));
  LPC_I2C0->I2SCLL   = I2SCLL_HS_SCLL;
  LPC_I2C0->I2SCLH   = I2SCLH_HS_SCLH;
#else
  I2CPADCFG &= ~((0x1<<0)|(0x1<<2));
  LPC_I2C0->I2SCLL   = I2SCLL_SCLL;
  LPC_I2C0->I2SCLH   = I2SCLH_SCLH;
#endif

  /* Install interrupt handler */
  NVIC_EnableIRQ(I2C0_IRQn);

  LPC_I2C0->I2CONSET = I2CONSET_I2EN;
  return;
}

/*****************************************************************************
** Function name:		I2C1Init
**
** Descriptions:		Initialize I2C controller as a master
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void I2C1Init( void ) 
{
  PCONP |= (1 << 19);
  // CPU Taktteiler fuer I2C1
  //PCLKSEL1_bit.PCLK_I2C1 = 0;   // CCLK/4
   //PCLKSEL1 |= (1<<20); //  1? // CCLK/1
  //PCLKSEL1_bit.PCLK_I2C1 = 2;   // CCLK/2
 //PCLKSEL1_bit.PCLK_I2C1 = 3;   // CCLK/8
  PCLKSEL1 &= ~(1<<6);   // CCLK/1
  PCLKSEL1 &= ~(1<<7);   // CCLK/1
  //PCLKSEL1 |=  (1<<6);   // CCLK/1
  //PCLKSEL1 |=  (1<<7);   // CCLK/1

#if 1
  /* set PIO0.0 and PIO0.1 to I2C1 SDA and SCL */
  /* function to 11 on both SDA and SCL. */
  PINSEL0 &= ~((0x3<<0)|(0x3<<2));
  PINSEL0 |= ((0x3<<0)|(0x3<<2));
  PINMODE0 &= ~((0x3<<0)|(0x3<<2));
  PINMODE0 |= ((0x2<<0)|(0x2<<2));		/* No pull-up no pull-down */
  PINMODE_OD0 |= ((0x01<<0)|(0x1<<1));	/* Open drain */
#endif
#if 0
  /* set PIO0.19 and PIO0.20 to I2C1 SDA and SCL */
  /* function to 11 on both SDA and SCL. */
  PINSEL1 &= ~((0x3<<6)|(0x3<<8));
  PINSEL1 |= ((0x3<<6)|(0x3<<8));
  PINMODE1 &= ~((0x3<<6)|(0x3<<8));
  PINMODE1 |= ((0x2<<6)|(0x2<<8));	/* No pull-up no pull-down */
  PINMODE_OD0 |= ((0x1<<19)|(0x1<<20));
#endif
 
  /*--- Clear flags ---*/
  LPC_I2C1->I2CONCLR = I2CONCLR_AAC | I2CONCLR_SIC | I2CONCLR_STAC | I2CONCLR_I2ENC;

  /*--- Reset registers ---*/
  LPC_I2C1->I2SCLL   = I2SCLL_SCLL;
  LPC_I2C1->I2SCLH   = I2SCLH_SCLH;

  /* Install interrupt handler */
  NVIC_EnableIRQ(I2C1_IRQn);

  LPC_I2C1->I2CONSET = I2CONSET_I2EN;
  return;
}

/*****************************************************************************
** Function name:		I2C2Init
**
** Descriptions:		Initialize I2C controller as a master
**
** parameters:			None
** Returned value:		None
** 
*****************************************************************************/
void I2C2Init( void ) 
{
  PCONP |= (1 << 26);

  /* set PIO0.10 and PIO0.11 to I2C2 SDA and SCL */
  /* function to 10 on both SDA and SCL. */
  PINSEL0 &= ~((0x03<<20)|(0x03<<22));
  PINSEL0 |= ((0x02<<20)|(0x02<<22));
  PINMODE0 &= ~((0x03<<20)|(0x03<<22));
  PINMODE0 |= ((0x02<<20)|(0x2<<22));	/* No pull-up no pull-down */
  PINMODE_OD0 |= ((0x01<<10)|(0x1<<11));
 
  /*--- Clear flags ---*/
  LPC_I2C2->I2CONCLR = I2CONCLR_AAC | I2CONCLR_SIC | I2CONCLR_STAC | I2CONCLR_I2ENC;

  /*--- Reset registers ---*/
  LPC_I2C2->I2SCLL   = I2SCLL_SCLL;
  LPC_I2C2->I2SCLH   = I2SCLH_SCLH;

  /* Install interrupt handler */
  NVIC_EnableIRQ(I2C2_IRQn);

  LPC_I2C2->I2CONSET = I2CONSET_I2EN;
  return;
}

/*****************************************************************************
** Function name:		I2CEngine
**
** Descriptions:		The routine to complete a I2C transaction
**						from start to stop. All the intermitten
**						steps are handled in the interrupt handler.
**						Before this routine is called, the read
**						length, write length, I2C master buffer,
**						and I2C command fields need to be filled.
**						see i2cmst.c for more details. 
**
** parameters:			I2C port number
** Returned value:		master state of current I2C port.
** 
*****************************************************************************/
uint32_t I2CEngine( uint32_t portNum ) 
{
  /*--- Issue a start condition ---*/
  LPC_I2C[portNum]->I2CONSET = I2CONSET_STA;	/* Set Start flag */

  I2CMasterState[portNum] = I2C_BUSY;	

  while ( I2CMasterState[portNum] == I2C_BUSY )
  {
	if ( timeout[portNum] >= MAX_TIMEOUT )
	{
	  I2CMasterState[portNum] = I2C_TIME_OUT;
	  break;
	}
	timeout[portNum]++;
  }
  LPC_I2C[portNum]->I2CONCLR = I2CONCLR_STAC;

  return ( I2CMasterState[portNum] );
}

/******************************************************************************
**                            End Of File
******************************************************************************/
void eep24c_wr(uint16_t adr16,uint16_t dat16){
//32k=>2048x16bit
	SLAVE_ADR=0xa0;
	  I2CWriteLength[PORT_USED] = 5;
	   I2CReadLength[PORT_USED] = 0;
	   I2CMasterBuffer[PORT_USED][0] = SLAVE_ADR;
	   I2CMasterBuffer[PORT_USED][1] = 	(adr16>>8);	/* address */
	   I2CMasterBuffer[PORT_USED][2] = (adr16&0xff);	/* adr2 Data0 */
	   I2CMasterBuffer[PORT_USED][3] =(dat16>>8);	 		/* Data1 */
	   I2CMasterBuffer[PORT_USED][4] =(dat16&0xff);		/* Data0 */
	   I2CEngine( PORT_USED );
	   _delay_ms(200);
//	   I2CStop(PORT_USED);
}

uint16_t eep24c_rd(uint16_t adr16){
//32k=>2048x16bit
	SLAVE_ADR=0xa0;
	   I2CWriteLength[PORT_USED] = 3;
	   I2CReadLength[PORT_USED] = 6;
	   I2CMasterBuffer[PORT_USED][0] = SLAVE_ADR;
	   I2CMasterBuffer[PORT_USED][1] =(adr16>>8); 		// address //
	   I2CMasterBuffer[PORT_USED][2] =(adr16&0xff);		// address //
	   I2CMasterBuffer[PORT_USED][3] = SLAVE_ADR | RD_BIT;
	   I2CEngine( PORT_USED );
	   _delay_ms(200);
	   return ((I2CSlaveBuffer[PORT_USED][0]<<8)|(I2CSlaveBuffer[PORT_USED][1]));
}

uint8_t  ds1307_rd(){
	uint8_t ssec=61;
	SLAVE_ADR=0xd0;
	   I2CWriteLength[PORT_USED] = 2;
	   I2CReadLength[PORT_USED] = 7;
	   I2CMasterBuffer[PORT_USED][0] = SLAVE_ADR;
	   I2CMasterBuffer[PORT_USED][1] =0; 		// address //
	   I2CMasterBuffer[PORT_USED][2] = SLAVE_ADR | RD_BIT;
	   I2CEngine( PORT_USED );
//		ssec=(((I2CSlaveBuffer[PORT_USED][0] & 0b11110000)>>4)*10)+(I2CSlaveBuffer[PORT_USED][0] & 0b00001111);
		ssec=I2CSlaveBuffer[PORT_USED][0];
/*		min=(((I2CSlaveBuffer[PORT_USED][1] & 0b11110000)>>4)*10)+(I2CSlaveBuffer[PORT_USED][1] & 0b00001111);
		hou=(((I2CSlaveBuffer[PORT_USED][2] & 0b11110000)>>4)*10)+(I2CSlaveBuffer[PORT_USED][2] & 0b00001111);
		WDY=(((I2CSlaveBuffer[PORT_USED][3] & 0b11110000)>>4)*10)+(I2CSlaveBuffer[PORT_USED][3] & 0b00001111);
		DAY=(((I2CSlaveBuffer[PORT_USED][4] & 0b11110000)>>4)*10)+(I2CSlaveBuffer[PORT_USED][4] & 0b00001111);
		MON=(((I2CSlaveBuffer[PORT_USED][5] & 0b11110000)>>4)*10)+(I2CSlaveBuffer[PORT_USED][5] & 0b00001111);
		YEA=(((I2CSlaveBuffer[PORT_USED][6] & 0b11110000)>>4)*10)+(I2CSlaveBuffer[PORT_USED][6] & 0b00001111);
*/
_delay_ms(200);
return(ssec);
}

void ds1307_wr(uint8_t sec){
	SLAVE_ADR=0xd0;
	   I2CWriteLength[PORT_USED] = 3;
	   I2CReadLength[PORT_USED] = 0;
	   I2CMasterBuffer[PORT_USED][0] = SLAVE_ADR;
	   I2CMasterBuffer[PORT_USED][1] =0; 		// address //
	   I2CMasterBuffer[PORT_USED][2] =sec;//((sec/10)<<4)|(sec%10);
	   I2CEngine( PORT_USED );
_delay_ms(200);
}

//=====================================
#define REG_PID         0x0a    /* Product ID MSB */
#define OV7670_ADDR     0x42
 void   i2c_clearbuffers(void){
	 uint8_t ix;
	 for ( ix = 0; ix < 10; ix++ ){
	 	I2CSlaveBuffer[PORT_USED][ix] = 0x00;
	 }

 }
uint8_t ov7670_get(uint8_t addr)
{
PORT_USED=1;
i2c_clearbuffers();
    I2CWriteLength[PORT_USED] = 2;
    I2CReadLength[PORT_USED] = 0;
    I2CMasterBuffer[PORT_USED][0] = OV7670_ADDR;   /* i2c address */
    I2CMasterBuffer[PORT_USED][1] = addr;          /* key */

    I2CEngine(PORT_USED);

    _delay_ms(1);

    i2c_clearbuffers();
    I2CWriteLength[PORT_USED] = 0;
    I2CReadLength[PORT_USED] = 1;
    I2CMasterBuffer[PORT_USED][0] = OV7670_ADDR | RD_BIT;

    //while (I2CEngine(PORT_USED) == I2CSTATE_SLA_NACK);
I2CEngine(PORT_USED);
_delay_ms(100);
    return I2CSlaveBuffer[PORT_USED][0];
}

void i2c1_scan(u8 startadr){
u8 ix,iz;
for(ix=startadr;ix<255;ix++){
lgh(1,1,ix);li(ic2_scan_found);
led1_tog;
ic2_scan_found=2;
	   I2CWriteLength[PORT_USED] = 1;
	   I2CReadLength[PORT_USED] = 0;
	   I2CMasterBuffer[PORT_USED][0] = ix;
	    I2CEngine(PORT_USED);
if(ic2_scan_found==0){
iz++;
lgh(iz,10,ix);
}
_delay_ms(200);}
}

#endif /* end __I2C_H */
/****************************************************************************
**                            End Of File
*****************************************************************************/
